-- CribGroup table which keeps track of the group name and description.
CREATE TABLE CribGroup (
    GroupId INT IDENTITY(1,1) PRIMARY KEY NOT NULL,
    Name VARCHAR(50) NOT NULL,
    Description VARCHAR(255)
);
GO

-- Junction table for CribGroups and Employees, allows a many to many relationship.
CREATE TABLE CribGroupEmployeeJunction (
    Id INT IDENTITY(1,1) PRIMARY KEY NOT NULL,
    GroupId INT NOT NULL,
    EmployeeId VARCHAR(25) NOT NULL,  
);
GO

-- Junction table for CribGroups and Cribs, allows a many to many relationship.
CREATE TABLE CribGroupCribJunction (
    Id INT IDENTITY(1,1) PRIMARY KEY NOT NULL,
    GroupId INT NOT NULL,
    CribId INT NOT NULL,
);
GO

-- Following types are so Table Valued Parameters can be sent for performance to hopefully avoid timeout.
create type StringListType as TABLE
(
	StringValue varchar(max)
);
GO

create type IntListType as TABLE
(
	IntValue int
);
GO

create type StringIntListType as TABLE
(
	StringValue varchar(max),
	IntValue int
);
GO

-- Procedure to add either employees or cribs to a group and add access for the employees to those cribs.
--  EmployeeList - Single column table of EmployeeIds to affect.
--  GroupList - Single column talbe of GroupIds for the employees/cribs.
--  CribList - Single column table of CribIds to affect.
--  MergeList - Two column table of (EmployeeId, CribId) or every combination to add to EmployeeCrib, sending this in to hopefully avoid timeout.
--  AffectingEmployees - Single digit number (0 = false, 1 = true) shows if either employees or cribs should be added to a group.
--  EmployeeCribEffects - Int output should return the number of rows being inserted into EmployeeCrib.
CREATE Procedure sp_AddToCribGroup
@EmployeeList StringListType READONLY,
@GroupList IntListType READONLY,
@CribList IntListType READONLY,
@MergeList StringIntListType READONLY,
@AffectingEmployees bit,
@EmployeeCribEffects int output
as
begin
begin Transaction;
begin try
	
	if @AffectingEmployees = 1
	begin		
		insert into CribGroupEmployeeJunction (EmployeeId, GroupId) select e.StringValue, g.IntValue from @EmployeeList e cross join @GroupList g;
	END
	else
	begin
	
		insert into CribGroupCribJunction (CribId, GroupId) select c.IntValue, g.IntValue from @CribList c cross join @GroupList g;
	end
		
	INSERT INTO EmployeeCrib (EmployeeID, Crib, CribAccessOption)
	SELECT m.StringValue, m.IntValue, 1
	FROM @MergeList m
	LEFT JOIN EmployeeCrib on EmployeeCrib.EmployeeID = m.StringValue and EmployeeCrib.Crib = m.IntValue
	WHERE EmployeeCrib.EmployeeID is null and EmployeeCrib.Crib is null;

	select @EmployeeCribEffects = @@ROWCOUNT;
	
	commit;
end try
begin catch
	
	declare @error int, @message varchar(4000), @xstate int;
    select @error = ERROR_NUMBER(), @message = ERROR_MESSAGE(), @xstate = XACT_STATE();
	rollback;
	raiserror('sp_AddToCribGroup: %d: %s', 16, 1, @error, @message) ;

end catch;
end;
GO

-- Procedure to Remove either employees or cribs to a group and remove access for the employees to those cribs.
--  EmployeeList - Single column table of EmployeeIds to affect.
--  GroupList - Single column talbe of GroupIds for the employees/cribs.
--  CribList - Single column table of CribIds to affect.
--  MergeList - Two column table of (EmployeeId, CribId) or every combination to remove from EmployeeCrib, sending this in to hopefully avoid timeout.
--  AffectingEmployees - Single digit number (0 = false, 1 = true) shows if either employees or cribs should be removed from a group.
--  EmployeeCribEffects - Int output should return the number of rows being removed from EmployeeCrib.
CREATE Procedure sp_RemoveFromCribGroup
@EmployeeList StringListType READONLY,
@GroupList IntListType READONLY,
@CribList IntListType READONLY,
@MergeList StringIntListType READONLY,
@AffectingEmployees bit,
@EmployeeCribEffects int output
as
begin
begin Transaction;
begin try
	
	if @AffectingEmployees = 1
	begin
		delete from CribGroupEmployeeJunction where EmployeeId in (select e.StringValue from @EmployeeList e) and GroupId in (select g.IntValue from @GroupList g);		
	END
	else
	begin
		delete from CribGroupCribJunction where CribId in (select c.IntValue from @CribList c) and GroupId in (select g.IntValue from @GroupList g);				
	end

	delete from EmployeeCrib where exists 
	(select 1 from @MergeList m where EmployeeCrib.EmployeeID = m.StringValue and EmployeeCrib.Crib = m.IntValue and not exists
	(select 1 from CribGroupEmployeeJunction ej join CribGroupCribJunction cj on ej.GroupId = cj.GroupId where ej.EmployeeId = m.StringValue and cj.CribId = m.IntValue));
	 	 
	select @EmployeeCribEffects = @@ROWCOUNT;
	
	commit;
end try
begin catch
	
	declare @error int, @message varchar(4000), @xstate int;
    select @error = ERROR_NUMBER(), @message = ERROR_MESSAGE(), @xstate = XACT_STATE();
	rollback;
	raiserror('sp_AddToCribGroup: %d: %s', 16, 1, @error, @message) ;

end catch;
end;
GO